//*******************************************************************
// File name: embed.c
//
//    Source file for creating embedded windows.  Provides the 
//    following functions:
//
// Functions:
// 
//    LibMain()          - used for DLL initialization
//    EmbedWindowProc()  - the window procedure for embedded windows
//    DrawFace()         - used to paint the Face bitmap
//    fGetFaceStatus()   - determines if the current ew is a face
//    hCreateBmpHandle() - creates an HBITMAP for EWM_RENDER
//    ptGetErrorSize()   - gets the extents of error msg text
//
// Development Team: Scott Skorupa
//
// Written by Microsoft Product Support Services, Windows Developer Support.
// Copyright (C) 1992 Microsoft Corporation.  All rights reserved.
//*******************************************************************

#include <windows.h>
#include "ew.h"             // embedded window constants
#include "embed.h"  
#include "globals.h"


//*******************************************************************
// LibMain()
//
// Purpose:
//
//     LibMain is called when EMBED.DLL is loaded.  This routine 
//     performs general initialization, such as loading info from
//     the string table, loading the face bitmap, and registering
//     the embedded window class.
//
// Parameters:
//     
//     HANDLE hInst       - Instance handle
//  
//     WORD wDataSeg      - Contains the value of the library DS
//
//     WORD cbHeapSize    - Specifies the size of the heap defined
//                          in the module-definition file.
// 
//     LPSTR lpszCmdLine  - Points to a NULL terminated string that
//                          specifies command line information.
//
// Return Value:
//     1 if successful
//     0 Otherwise
//
// Comments:
//
// History:      Date      Author      Reason
//               5/28/92   scottsk     created
//
//*******************************************************************
  
int FAR PASCAL LibMain(HANDLE hInst,WORD wDataSeg,WORD cbHeapSize,LPSTR lpszCmdLine)
{
    WNDCLASS wc;
    char szClassName[MAXCHARS];
    
    ghInst = hInst;
   
    // load the bitmap that is to be displayed in the embedded window   
    // if it fails, we need to return failure.
    
    if (!(ghFaceBmp = LoadBitmap (ghInst,"face")))
        return 0;   // return failure
    
    // load the error string.
        
    if (!LoadString(ghInst, IDS_ERROR, gszError, ERRORSIZE))
        return 0;   // return failure
    
    // load the class name for the embedded window
        
    if (!LoadString(ghInst, IDS_CLASS, szClassName, MAXCHARS))
        return 0;   // return failure

    wc.style = CS_GLOBALCLASS;                  // need this style so the DLL can find
                                                // the window class.
    wc.lpfnWndProc = EmbedWindowProc;
    wc.cbClsExtra = 0;
    wc.cbWndExtra = sizeof(HANDLE);             // to store the handle to the "window" global
                                                // information
    wc.hInstance = hInst;
    wc.hIcon = NULL;
    wc.hCursor = LoadCursor(NULL, IDC_ARROW);
    wc.hbrBackground = (HBRUSH)(COLOR_WINDOW+1);
    wc.lpszMenuName = (LPCSTR) NULL;
    wc.lpszClassName = szClassName;    
    
    if (!RegisterClass(&wc))
        return 0;   // return failure

    return 1;       // return success
}

//*******************************************************************
// EmbedWindowProc()
//
// Purpose:
//
//     This is the window procedure for the embedded windows.
//
// Parameters:
//     
//     HWND hWnd          - Handle to the embedded window
//  
//     UINT msg           - Contains the message for the window
//
//     WPARAM wParam      - message specific
// 
//     LPARAM lParam      - message specific
//
// Return Value:
//     message specific
//
// Comments:
//
// History:      Date      Author      Reason
//               5/28/92   scottsk     created
//               6/10/92   scottsk     fixed Memory bug and added
//                                     the check for Major and Minor
//                                     version numbers.
//
//*******************************************************************

LRESULT CALLBACK EmbedWindowProc (HWND hWnd,UINT msg,WPARAM wParam,LPARAM lParam)
{
    PAINTSTRUCT ps;
    HDC hDC;
    HANDLE hText,hGlobalData;
    LPSTR lpText;
    RENDERINFO FAR *ri;
    POINT pt;
    POINT ptSize;
    EWDATA FAR *lped;
    GLOBALDATA FAR *lpGlobalData;
    RECT rect;


    switch (msg) {
    
        // the help engine sends this notification when it needs to know the
        // size of the embedded window

        case EWM_QUERYSIZE:
            // lParam contains a pointer to a POINT structure
            // wParam contains the target DC
        
            // this checks to see if the embedded window contains a face
            
            if (fGetFaceStatus(hWnd))                   { 
                ptSize.x = 32;
                ptSize.y = 32;
                DPtoLP ((HDC)wParam, &ptSize, 1);
                ((LPPOINT)lParam)->x = ptSize.x ;
                ((LPPOINT)lParam)->y = ptSize.y ;
                }
            else {
                pt = ptGetErrorSize((HDC)wParam);
                ((LPPOINT)lParam)->x = pt.x;
                ((LPPOINT)lParam)->y = pt.y;
                }
                
            return 1;  // shows that this notifications was processed.
            
            
        // this notification is sent to the embedded window when the help
        // engine needs information to be copied to the clipboard, or 
        // printed to the printer.    

        case EWM_RENDER:
        
            // lParam contains a pointer to a RENDERINFO structure
            // wParam contains the format that the help engine wants
            
            ri = (RENDERINFO FAR *)lParam;
            
            switch (wParam) {
                case CF_TEXT:
                    if (hText = GlobalAlloc (GMEM_MOVEABLE | GMEM_DISCARDABLE,
                                  lstrlen(gszError)+1)) {
                        lpText = (LPSTR) GlobalLock(hText);
                        lstrcpy (lpText,gszError);
                        GlobalUnlock(hText);
                        return hText;
                        }
                    else 
                        return NULL;
               
                case CF_BITMAP:
                    return hCreateBmpHandle(hWnd, ri->hdc);
                }

            break;
            
        case WM_CREATE:
            // lParam contains a pointer to a CREATESTRUCT structure
            // wParam is not used
        
        
            // get a pointer to the EWDATA structure
            lped = (EWDATA FAR *)((CREATESTRUCT FAR *)lParam)->lpCreateParams;
            
            
            // if "major" version doesn't match, fail creation
            if (lped->idMajVersion != MAJORVERSION)
                return -1;
                
            // if "minor" version indicates an earlier version of the help
            // engine, fail the creation.    
            if (lped->idMinVersion < MINORVERSION)
                return -1;    
                
            // allocate memory that is specific to the window being created. 
            
            if (!(hGlobalData = GlobalAlloc (GMEM_MOVEABLE | GMEM_DISCARDABLE, sizeof(GLOBALDATA))))
                return -1;    // failure to allocate needed memory, abort creation
            
            SetWindowWord (hWnd, 0, hGlobalData);
            
            // get a pointer to the memory so it can be initialized
            
            lpGlobalData = (GLOBALDATA FAR *) GlobalLock (hGlobalData);
            
            // fill out the GLOBALDATA structure with the EWDATA information.
            // this info is not used in this sample, but this process is presented
            // for demonstrative purposes.
            
            lpGlobalData->ed.idMajVersion = lped->idMajVersion;
            lpGlobalData->ed.idMinVersion = lped->idMinVersion;
            lpGlobalData->ed.hfs = lped->hfs;
            lpGlobalData->ed.coFore = lped->coFore;
            lpGlobalData->ed.coBack = lped->coBack;                       
            lstrcpy (lpGlobalData->chFileName, lped->szFileName);
            lstrcpy (lpGlobalData->chAuthorData, lped->szAuthorData);
           
            // check the "author data".  If it is equal to face, then set the face
            // flag.  Otherwise clear the face flag, and set a border around the window.
            // We only want the border when we are printing an error message.
            
            if (!lstrcmp(lped->szAuthorData,(LPSTR)"face")) 
                lpGlobalData->gfFace = TRUE;
            else {
                lpGlobalData->gfFace = FALSE;
                SetWindowLong(hWnd, GWL_STYLE, GetWindowLong(hWnd, GWL_STYLE) | WS_BORDER);                
                }
            
            // be nice and unlock the memory.
                
            GlobalUnlock (hGlobalData);    

            return 0L;   // return success
            
        case WM_DESTROY:
            
            // Get the memory associated with this embedded window, and free it.
            
            hGlobalData = GetWindowWord(hWnd,0);
            GlobalFree(hGlobalData);
            return 0L;

        case WM_LBUTTONDOWN:
            
            // Just force a WM_PAINT message.  The values are recalculated
            // in the processing of the WM_PAINT message.  This really doesn't 
            // accomplish anything, except to demonstrate that mouse messages
            // are received by embedded windows.

            InvalidateRect(hWnd, NULL, TRUE);
            break;

        case WM_PAINT:
            hDC = BeginPaint (hWnd, &ps);
            
            // if this window contains a face, draw it.  Otherwise display an
            // error message.
            
            if (fGetFaceStatus(hWnd)) 
                 DrawFace (hDC);  
            else {
                pt = ptGetErrorSize(hDC);  // gets the "extents" of the error message
                rect.left = 0;
                rect.top = 0;
                rect.right = pt.x;
                rect.bottom = pt.y;                 
                DrawText(hDC,gszError,lstrlen(gszError),&rect,DT_LEFT);
                }
                
            EndPaint(hWnd,&ps);
            
            return 0L;
        }  // end of switch
        
    return DefWindowProc(hWnd, msg, wParam, lParam);

}


//*******************************************************************
// DrawFace()
//
// Purpose:
//
//     Draws the face bitmap.
//
// Parameters:
//     
//     HDC hDC            - Handle to the device context
//
// Return Value:
//     none
//
// Comments:
//
// History:      Date      Author      Reason
//               5/28/92   scottsk     created
//
//*******************************************************************

void DrawFace(HDC hDC)

{
    HDC hDC2;
    POINT ptSize;
    HBITMAP hOldBmp;

    // create "scratch space"
    
    hDC2 = CreateCompatibleDC(hDC);
    hOldBmp = SelectObject(hDC2, ghFaceBmp);
    SetMapMode(hDC2, GetMapMode(hDC));
    
    // the size of the bitmap is 32x32 pixels.  Since the mapping mode
    // of the DC is not known, we need to convert the size to logical
    // points.
    
    ptSize.x = 32;
    ptSize.y = 32;
    DPtoLP(hDC,&ptSize,1);
    
    // draw the object
    
    BitBlt (hDC, 0, 0, ptSize.x, ptSize.y, hDC2, 0, 0, SRCCOPY);
    SelectObject (hDC2, hOldBmp);
    DeleteDC (hDC2);
    
}    

//*******************************************************************
// fGetFaceStatus()
//
// Purpose:
//
//     Gets the data specific to this particular embedded window and
//     returns the value of the face flag.
//
// Parameters:
//     
//     HWND hWnd            - Handle to the embedded window
//
// Return Value:
//     TRUE if the embedded window contains a face
//     FALSE if the embedded window is an "error"
//
// Comments:
//
// History:      Date      Author      Reason
//               5/28/92   scottsk     created
//
//*******************************************************************

BOOL fGetFaceStatus(HWND hWnd)

{
    BOOL fReturn = FALSE;
    HANDLE hGlobalData;
    GLOBALDATA FAR *lpGlobalData;

    // get a pointer to the data
    
    hGlobalData = GetWindowWord(hWnd,0);
    lpGlobalData = (GLOBALDATA FAR *)GlobalLock(hGlobalData);
    
    // set the return value to equal the face flag
    
    fReturn = lpGlobalData->gfFace;
    
    // lets be good, and unlock the memory
    
    GlobalUnlock (hGlobalData);
    
    
    return fReturn;
    
}    
    
    
//*******************************************************************
// hCreateBmpHandle()
//
// Purpose:
//
//     creates a copy of the face bitmap or an "error" bitmap, and 
//     returns a handle to this copy of the bitmap.
//
// Parameters:
//     
//     HWND hWnd            - Handle of the embedded window
//     HDC hDCHelpEngine    - the reference DC passed by the help engine
//
// Return Value:
//     handle to the bitmap to be returned to the help engine
//
// Comments:
//
// History:      Date      Author      Reason
//               5/28/92   scottsk     created
//
//*******************************************************************
    
HBITMAP hCreateBmpHandle(HWND hWnd, HDC hDCHelpEngine)

{
HDC hDC,hDC2;
HBITMAP hBitmap,hOldBitmap;
POINT pt;
RECT rc;
HBRUSH hBrush;
RECT rect;

    // create a "temporary" DC and initializes

    hDC = CreateCompatibleDC(NULL);
    SetMapMode(hDC, GetMapMode(hDCHelpEngine));
    
    //  Create a bitamp, and select it into the temporary DC.
    SendMessage(hWnd, EWM_QUERYSIZE, hDC, (LONG)(LPPOINT)&pt);
    hBitmap = CreateCompatibleBitmap (hDC, pt.x, pt.y);
    hOldBitmap = SelectObject (hDC, hBitmap);
    
    if (fGetFaceStatus (hWnd))    {     // is it a face
      
        // need to get another DC, and initialize it with the "face" bitmap
        
        hDC2 = CreateCompatibleDC(NULL);
        SetMapMode(hDC2, GetMapMode(hDCHelpEngine));
        SelectObject(hDC2, ghFaceBmp);
        
        // draw the face bitmap into the original "temporary" DC
        BitBlt(hDC, 0, 0, pt.x, pt.y, hDC2, 0, 0, SRCCOPY);
        
        // clean up
        DeleteDC (hDC2);
        }
    else { 
        
        // an error bitmap needs to be drawn.
        
        // clear the background of the bitmap
        
        if (hBrush = CreateSolidBrush(GetBkColor(hDC))) {
            FillRect(hDC, &rc, hBrush);
            DeleteObject(hBrush);
            }

        // draw a border
                    
        Rectangle(hDC,0,0,pt.x,pt.y);
        
        // draw the text
        rect.top = 0;
        rect.left = 0;
        rect.bottom = pt.y;
        rect.right = pt.x;
        DrawText(hDC,gszError,lstrlen(gszError),&rect,DT_SINGLELINE | DT_VCENTER | DT_CENTER);
        }

      // get a handle to the bitmap that needs to be returned.
      
      hBitmap = SelectObject(hDC, hOldBitmap);
      
      // clean up
      
      DeleteDC (hDC);
      
      return hBitmap;
}    

//*******************************************************************
// ptGetErrorSize()
//
// Purpose:
//
//     Determines the dimensions needed for the window to display
//     an error message.
//
// Parameters:
//     
//     HDC hDC            - Handle to the device context
//
// Return Value:
//     A POINT structure containing the height and width needed for
//     the window.
//
// Comments:
//
// History:      Date      Author      Reason
//               5/28/92   scottsk     created
//
//*******************************************************************
    
POINT ptGetErrorSize(HDC hDC)    
{
POINT pt;
RECT rect;

    // init the rect
    
    rect.top = 0;
    rect.bottom = 0;
    rect.right = 0;
    rect.left = 0;

    // get the dimensions
    
    DrawText(hDC,gszError,lstrlen(gszError), &rect, DT_LEFT  | DT_CALCRECT);   

    // set up a point structure, and add some white space
    
    pt.x = rect.right + EXTRAWIDTH;
    pt.y = rect.bottom + EXTRAHEIGHT;

    return pt;
    
}    
    

    
